<?php 
/**

  The Initial Developer of the Original Code is
  Matthieu  - http://www.programmation-facile.com/
  Portions created by the Initial Developer are Copyright (C) 2013
  the Initial Developer. All Rights Reserved.

  Contributor(s) :

 */
 
/**
 * @file Master.php
 * @desc Init the website, create header, footer, include classes
 * 
 * @author Matthieu
 *
 *
 * @version 0.1 
 * @usage: 

$s_homeDirectory = "";
require_once ($s_homeDirectory."include/classes/Master.php");
$o_Master = Master::GetInstance();
$o_Master->setInit($_POST['come']);
$o_Master->newPage($s_homeDirectory);
$o_Master->addHeader();

Votre code HTML / javascript de la page

$o_Master = Master::GetInstance()->addFooter("Accueil Delfiweb");
 */
 
/**
 * @note : 
 * 			- this class is a singleton.	
 */
class Master 
{
	
	/* constantes */
	private static $o_instance = NULL; // unique instance
	
	
	/* configuration */
	private $s_homeDirectory; 
	private $s_Domain;
	private $b_addHeader;
	private $b_addFooter;
	private $b_modeTest;
	
	
	
	// ---------------------------------- __construct() ----------------------------------------------------------
   /**
	* @brief Constructeur de la classe. vide pour l'instant.
	* @param -
	*/
	private function __construct()
	{
		$this->b_addHeader = false;
		$this->b_addFooter = false;
		$this->s_Domain = "http://www.delfiweb.com/";
		self::$o_instance = $this;
		
		// fermeture temporaire du site pour maintenance
		/*header("Status: 302 Moved Permanently");
		header("Location: http://www.delfiweb.com/maintenance.php");
		exit();*/
	}


	// -----------------------------------------------------------------------------------------------------------
	// -------------------------------------- M�thodes publiques  ------------------------------------------------
	// -----------------------------------------------------------------------------------------------------------
	
	
	// ---------------------------------- setInit() ----------------------------------------------------------
	/**
	* @brief Inititalisation du site
	* 
	* @param $b_value false -> affichage du header et du footer
	* @param $b_testtrue -> activation du mode maintenance
	* @return
	*/
	public function setInit($b_value=false, $b_test=false)
	{
		$this->b_addHeader = $b_value;
		$this->b_addFooter = $b_value;
		$this->b_modeTest = $b_test;
	}
	
	
	// ---------------------------------- newPage() ----------------------------------------------------------
	/**
	* @brief Cosntruction d'une page web, inclusion des classes php
	* 
	* @param (string) root path
	* @return
	*/
	public function newPage( $s_Path="" )
	{
		$this->s_homeDirectory = $s_Path;
		
		/*--- on empeche le navigateur de mettre la page en cache ---*/
		clearstatcache(); // efface les donnes php en cache
		$CachePlus = "";
		$CachePlus = ", pre-check=0, post-check=0, max-age=0";
		$now = gmdate('D, d M Y H:i:s') . ' GMT';
		header("Expires: $now");
		//header("Expires: Mon, 26 Jul 1997 05:00:00 GMT"); 
		header("Last-Modified: $now");
		header("Cache-Control: no-cache, must-revalidate".$CachePlus);
		header("Pragma: no-cache");	

		// include de toutes les classes du site
		require_once($this->s_homeDirectory."include/classes/Mysql.php");
		require_once($this->s_homeDirectory."include/classes/Guestbook.php");
		require_once($this->s_homeDirectory."include/classes/Utils.php");
		require_once($this->s_homeDirectory."include/classes/Reference.php");		
		require_once($this->s_homeDirectory."include/classes/Debugger.php");
		
		$this->createReference();// pour le referencement du site flash
	}
	
	
	// ---------------------------------- addHeader() ----------------------------------------------------------
	/**
	* @brief Affiche le header de la page web
	* 
	* @param $s_title le titre de la page
	* @return
	*/
	public function addHeader($s_title="Delfiweb - Ressources Flash, Framework AS2, AS3, HAXE")
	{
		$s_titlePage = utf8_encode($s_title);
		
		if($this->b_addHeader == false)
		{
			$s_homeDirectory = $this->s_homeDirectory;
			if( $this->b_modeTest==false ) require_once($this->s_homeDirectory."template/header.php");
			else require_once($this->s_homeDirectory."template/header_test.php");
			$this->b_addHeader = true;
		}
		else echo "<script type=\"text/javascript\"> document.title=('".addslashes($s_titlePage)."') </script>";
	}
	
	
	
	// ---------------------------------- addFooter() ----------------------------------------------------------
	/**
	* @brief Affiche le footer de la page
	* 
	* @param $s_pageName utile pour ajouter un suivi avec google analytics par nom de page
	* @return
	*/
	public function addFooter($s_pageName="Delfiweb - No Name")
	{
		if($this->b_addFooter == true) return;
		$s_homeDirectory = $this->s_homeDirectory;
		if( $this->b_modeTest==false ) require_once($this->s_homeDirectory."template/footer.php");
		else require_once($this->s_homeDirectory."template/footer_test.php");
		$this->b_addFooter = true;
	}
	
	
	// ---------------------------------- addReference() ----------------------------------------------------------
	/**
	* @brief Ajout du texte de referencement pour google et pour une application flash.
	* 
	* @return
	*/
	public function addReference()
	{
		/*--- pour le referencement sur les sites fr ---*/
		require_once($this->s_homeDirectory."include/page_content/contentXML.htm");
	}
		

	// -----------------------------------------------------------------------------------------------------------
	// --------------------------------------------- Getter ------------------------------------------------------
	// -----------------------------------------------------------------------------------------------------------
	
	
	
	/**
	 * @brief redefinition de la methode __get afin de determiner comment la classe doit ragir lorsqu'on appelle une de ses proprietes
	 * Un Getter / SEtter en php !
	 * 
	 * @param (string)	le nom de la propriete a retourner
	 * @return le contenu de la propriete
	 */
	public function __get($s_prop)
	{
		switch($s_prop)
		{
			case 'homeDir' :
				return	$this->s_homeDirectory;
			
			case 'init' : 
				return	$this->b_addHeader;
			
			case 'domain' : 
				return	$this->s_Domain;
				
			default :
				$s_message = 'la propriété '.$s_prop.' n\'existe pas dans la classe '.__CLASS__.'</br>'; // affiche une erreur
				echo $s_message;
				return $s_message;
		}	
	}


	
	// -----------------------------------------------------------------------------------------------------------
	// -------------------------------------------- Setter -------------------------------------------------------
	// -----------------------------------------------------------------------------------------------------------
	
	

	/**
	 * @brief redefinition de la methode __set afin de determiner comment la classe doit ragir lorsqu'on appelle une de ses attributs
	 * Un Getter / SEtter en php !
	 * 
	 * @param (string)	le nom de la propriété à modifier
	 * @param (-)	la nouvelle valeur de la propriété
	 * @return -
	 */
	public function __set($s_prop, $value)
	{	
		switch($s_prop)
		{
			case 'homeDir' :
				$this->s_homeDirectory = $value;
				break;

			default :
				echo 'SETTER la propri�t� '.$s_prop.' n\'existe pas dans la classe '.__CLASS__.'</br>'; // affiche une erreur
		}
		
	}


	// ---------------------------------- GetInstance() ----------------------------------------------------------
	/**
	* @return (Master) l'instance unique du Singleton
	*/
	public static function GetInstance() 
	{
		if ( self::$o_instance == NULL ) self::$o_instance = new Master();
		
		return self::$o_instance;
	}

		
		
	
	// -----------------------------------------------------------------------------------------------------------
	// ---------------------------------------- M�thodes priv�es  ------------------------------------------------
	// -----------------------------------------------------------------------------------------------------------

	
	
	// ---------------------------------- createReference() ----------------------------------------------------------
	/**
	* @brief Creation du fichier html de referencement pour google.
	* Utilisation de fichiers xml
	* 
	* @param
	* @return
	*/
	private function createReference()
	{
		/* référencement */
		$i_ref =  rand(1, 333);// mise a jour aleatoire du fichier html
		
		//$i_ref  = 123;
		if ( $i_ref == 123 )
		{
			// recupere les fichiers xml
			$a_xmlTxt = array($this->s_homeDirectory."xml/content/fr/accueil.xml", $this->s_homeDirectory."xml/content/fr/news.xml", $this->s_homeDirectory."xml/content/fr/framework.xml", $this->s_homeDirectory."xml/content/fr/projets.xml", $this->s_homeDirectory."xml/content/fr/tutoriaux.xml"); // "xml/content/fr/contact.xml", $this->s_homeDirectory."xml/content/fr/partenaires.xml"
			
			// utilisation de la classe dediee
			$o_reference = new Reference($a_xmlTxt);
			
			$b_result = $o_reference->createFileHTML($this->s_homeDirectory."include/page_content/contentXML.htm");// creation du fichier html
			
			$s_adress = "contact@delfiweb.com";
			$s_subject = "Referencement automatique Delfiweb.com";
			
			// envoie d'un mail
			if( $b_result==true )
				mail($s_adress, $s_subject, "Fichier de referencement contentXML.htm cree avec succes.");
			else
				mail($s_adress, $s_subject, "Erreur lors de la creation du fichier de referencement.");
		}
	}
	
	
	
 	// -------------------------------------------------------------------------------------------------
	// ---------------------------------------- Divers  ------------------------------------------------
	// -------------------------------------------------------------------------------------------------

	
 	/**
	* @brief Destructeur de la classe
	* Utilise pour enregistrer des messages pour le debogage, fermer les fichier ouverts en écriture ou en lecture, réaliser des opérations
	* de nettoyage des objets utilisés...
	*
	* @param 
	* @return 
	*/
	public function __destruct() 
	{
 
 	}



	/**
	 * @brief Détermine comment la classe doit réagir lorsqu'elle est convertie en chaine de caracteres.
	 * 
	 * @usage   echo(myObject) ou print(myObject)
	 * 
	 * @return  la représentation de l'objet sous forme de cha�ne.
	 */	
	public function __toString() 
	{
		
		$s_classContent = "[Object ".__CLASS__."]<br />";
		// parcours toutes les propriétés de la classe et les renvoie	
		foreach($this as $prop => $value) 
		{
		 	$s_classContent .= "$prop => $value <br />";
		}
		 
		return $s_classContent;
   	}

}

?>