package application.app.view
{
	import flash.utils.getTimer;
	import flash.display.MovieClip;
	import flash.display.Stage;

	import application.app.model.RPGProxy;

	import flash.ui.Keyboard;
	import flash.events.KeyboardEvent;

	import flash.display.Sprite;

	import application.app.ApplicationFacade;

	import org.puremvc.as3.interfaces.IMediator;
	import org.puremvc.as3.interfaces.INotification;
	import org.puremvc.as3.patterns.mediator.Mediator;

	/**
	 * Permet de construire notre menu et d'interagir avec nos composants graphiques.
	 * 
	 * @author matth
	 */
	public class ViewHeros extends Mediator implements IMediator
	{
		public static const NAME : String = "ViewHeros";
		
		private var oHeros : MovieClip;

		private var nScore : int;
		private var nLife : int;

		private var nTimeLastTouch : int;// timer dernier ennemis -- permet au héros de ne pas mourir trop vite !
		
		
		/* déplacement du héros */
		private var bMouveRight : Boolean;
		private var bMouveLeft : Boolean;
		private var bjump : Boolean;
		
		private var bHerosDead : Boolean;// verifie si le heros est toujours vivant
		
		

		public function ViewHeros(viewComponent : Object)
		{
			super(NAME, viewComponent);
		}


		/**
		 * Initialisation de l'interface utilisateur.
		 * 
		 */
		private function setupUi(bStart:Boolean) : void
		{
			// création et configuration du héro
			if( oHeros!= null )
				stageV.removeChild(oHeros);

			oHeros = new Heros();
			oHeros.gotoAndStop(0);
			oHeros.char.gotoAndStop(0);
			stageV.addChild(oHeros);
			
			nTimeLastTouch = 0;
			
			var oData:Object = new Object();
			oData.width = oHeros.width;
			oData.height = oHeros.height;
			oData.clip = oHeros;
			sendNotification(ApplicationFacade.HEROS_READY, oData);
			
			// si relance juste le niveau, ne pas réinitialiser le score et les vies.
			if(!bStart) return;
			
			// apprès héros ready, car ViewMap initialise certaines variables
			nScore = 0;
			sendNotification(ApplicationFacade.HEROS_SCORE, nScore );
			
			nLife = 3;
			sendNotification(ApplicationFacade.HEROS_LIFE, nLife );
		}

		/**
		 * Prépare les controles clavier pour déplacer le palet.
		 * 
		 */
		private function setupControls() : void
		{
			bHerosDead = bMouveRight = bMouveLeft = bjump = false;
			
			var oCasseBriqueProxy : RPGProxy = facade.retrieveProxy(RPGProxy.NAME) as RPGProxy;
			var oStageAppli:Stage = oCasseBriqueProxy.stage;
			
			// appuie sur une touche de votre clavier
			oStageAppli.addEventListener(KeyboardEvent.KEY_DOWN, onKeyDown);
			oStageAppli.addEventListener(KeyboardEvent.KEY_UP, onKeyUp);
		}
		
		
		public function stopControls() : void
		{
			var oCasseBriqueProxy : RPGProxy = facade.retrieveProxy(RPGProxy.NAME) as RPGProxy;
			var oStageAppli:Stage = oCasseBriqueProxy.stage;
			
			// appuie sur une touche de votre clavier
			oStageAppli.removeEventListener(KeyboardEvent.KEY_DOWN, onKeyDown);
			oStageAppli.removeEventListener(KeyboardEvent.KEY_UP, onKeyUp);
		}
			
		/**
		 * Déplace le palet en fonction de la touche appuyée.
		 * 
		 */
		private function moveHeros() : void
		{
			var oData:Object = new Object();
			oData.dirx = 0; 
			oData.diry = 0; 
				
			if (bjump == true )// mario saute			
				sendNotification(ApplicationFacade.HEROS_JUMP);
				
			if (bMouveLeft == true)
			{
				oHeros.gotoAndStop("gauche");// positionne dans la direction du déplacement
				oData.dirx = -1; 
				oData.diry = 0; 
				sendNotification(ApplicationFacade.MOVE_MAP, oData);
			}
			else if (bMouveRight == true)
			{
				oHeros.gotoAndStop("droite");// positionne dans la direction du déplacement
				oData.dirx = 1; 
				oData.diry = 0; 
				sendNotification(ApplicationFacade.MOVE_MAP, oData);
			}
			else // aucune touche pressée
			{
				oHeros.char.stop();
				return;
			}		
			
			if(oData.dirx != 0)	oHeros.char.play();			
							
		}

		
		/**
		 * Test de la touche relachée.
		 * 
		 */
		private function onKeyUp(e : KeyboardEvent) : void
		{
			// appuie sur la touche droite ou gauche;
			if ( e.keyCode == Keyboard.RIGHT ) bMouveRight = false;
				
			if ( e.keyCode == Keyboard.LEFT ) bMouveLeft = false;
				
			if ( e.keyCode == Keyboard.UP ) bjump = false;
		}


		/**
		 * Test de la touche appuyée.
		 * 
		 */
		private function onKeyDown(e : KeyboardEvent) : void
		{
			// appuie sur la touche droite ou gauche;
			if ( e.keyCode == Keyboard.RIGHT )	bMouveRight = true;
				
			if ( e.keyCode == Keyboard.LEFT )	bMouveLeft = true;
				
			if ( e.keyCode == Keyboard.UP ) bjump = true;
		}

		/**
		 * Mets à jour la position du Héros.
		 * Appel par un évènement.
		 * 
		 */
		private function updatePositionHeros(nX:Number, nY:Number) : void
		{
			if( nX > oHeros.x )
				sendNotification(ApplicationFacade.MOVE_BACKGROUND, 1 );
			else if( nX < oHeros.x )
				sendNotification(ApplicationFacade.MOVE_BACKGROUND, -1 );
			
			oHeros.x = nX;
			oHeros.y = nY;			
		}
		

		
		
		/**
		 * Le héros gagne un bonus.
		 * Met à jour sa vie.
		 * 
		 */
		private function updateScore(nValue : int) : void
		{
			nScore += nValue;
			sendNotification(ApplicationFacade.HEROS_SCORE, nScore );
		}
		
		
		/**
		 * Mise à jour du niveau de vie du héros.
		 * Il est touché par un ennemis.
		 * 
		 */
		private function updateLifeByEnnemis() : void
		{
			if( getTimer() < (nTimeLastTouch + 669) )	return;
			
			bHerosDead = true;
			nTimeLastTouch = getTimer();
			nLife-- ;
			// fin du jeu - game over si le héros n'a plus de vie !
			if( nLife  < 0)
			{
				nLife = 0;
				sendNotification(ApplicationFacade.COMMAND_GAME_OVER);
			}
			else	sendNotification(ApplicationFacade.COMMAND_GAME_OVER, RPGProxy.RESTART_LEVEL);
			
			oHeros.gotoAndStop("dead");
			sendNotification(ApplicationFacade.HEROS_LIFE, nLife );			
		}
		
		/**
		 * Transtypage du viewComponent en son type actuel.
		 * 
		 */
		protected function get stageV() : Sprite
		{
			return viewComponent as Sprite;
		}

		/**
		 * Notifications attendues.
		 * 
		 */
		override public function listNotificationInterests() : Array
		{
			return [ApplicationFacade.ON_ENTER_FRAME,
					ApplicationFacade.START_GAME,
					ApplicationFacade.RESTART_LEVEL,
					ApplicationFacade.UPDATE_POS_HEROS,
					ApplicationFacade.UPDATE_HEROS_SCORE,
					ApplicationFacade.ENNEMIS_TOUCH_HEROS];
		}

		/**
		 * Gestion des notifications.
		 * 
		 */
		override public function handleNotification(note : INotification) : void
		{
			switch (note.getName())
			{
				case ApplicationFacade.START_GAME:
					this.setupUi(true);
					this.setupControls();
					break;
				
				case ApplicationFacade.RESTART_LEVEL:
					this.setupUi(false);
					this.setupControls();
					break;
					
				case ApplicationFacade.UPDATE_POS_HEROS:
					var oData:Object = Object( note.getBody() );
					this.updatePositionHeros(oData.x, oData.y);
					break;
				
				case ApplicationFacade.ENNEMIS_TOUCH_HEROS:
					this.updateLifeByEnnemis();
					break;
				
				case ApplicationFacade.UPDATE_HEROS_SCORE:// le héros touche un item bonus type vie
					this.updateScore(  int( note.getBody() ) );
					break;
								
				case ApplicationFacade.ON_ENTER_FRAME:
					if(!bHerosDead)	this.moveHeros();
					break;	
			}
		}


	}
}
